"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Json(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    BODY_FIELD_NUMBER: _builtins.int
    EXPANDED_FIELD_NUMBER: _builtins.int
    MAX_EXPAND_DEPTH_FIELD_NUMBER: _builtins.int
    body: _builtins.str
    """Content to display."""
    expanded: _builtins.bool
    """Controls the initial expansion state of the json element.
    Is superseded by max_expand_depth if provided.
    """
    max_expand_depth: _builtins.int
    """The maximum depth to expand the JSON object."""
    def __init__(
        self,
        *,
        body: _builtins.str = ...,
        expanded: _builtins.bool = ...,
        max_expand_depth: _builtins.int | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_max_expand_depth", b"_max_expand_depth", "max_expand_depth", b"max_expand_depth"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_max_expand_depth", b"_max_expand_depth", "body", b"body", "expanded", b"expanded", "max_expand_depth", b"max_expand_depth"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__max_expand_depth: _TypeAlias = _typing.Literal["max_expand_depth"]  # noqa: Y015
    _WhichOneofArgType__max_expand_depth: _TypeAlias = _typing.Literal["_max_expand_depth", b"_max_expand_depth"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__max_expand_depth) -> _WhichOneofReturnType__max_expand_depth | None: ...

Global___Json: _TypeAlias = Json  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from streamlit.proto import ArrowData_pb2 as _ArrowData_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class MixedData(_message.Message):
    """Mixed data structure that allows JSON with embedded Arrow references"""

    DESCRIPTOR: _descriptor.Descriptor

    @_typing.final
    class ArrowBlobsEntry(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        KEY_FIELD_NUMBER: _builtins.int
        VALUE_FIELD_NUMBER: _builtins.int
        key: _builtins.str
        @_builtins.property
        def value(self) -> _ArrowData_pb2.ArrowData: ...
        def __init__(
            self,
            *,
            key: _builtins.str = ...,
            value: _ArrowData_pb2.ArrowData | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["value", b"value"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["key", b"key", "value", b"value"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    JSON_FIELD_NUMBER: _builtins.int
    ARROW_BLOBS_FIELD_NUMBER: _builtins.int
    json: _builtins.str
    """The main JSON structure with arrow references replaced by placeholders"""
    @_builtins.property
    def arrow_blobs(self) -> _containers.MessageMap[_builtins.str, _ArrowData_pb2.ArrowData]:
        """Map of arrow data blobs keyed by reference ID"""

    def __init__(
        self,
        *,
        json: _builtins.str = ...,
        arrow_blobs: _abc.Mapping[_builtins.str, _ArrowData_pb2.ArrowData] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["arrow_blobs", b"arrow_blobs", "json", b"json"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___MixedData: _TypeAlias = MixedData  # noqa: Y015

@_typing.final
class BidiComponent(_message.Message):
    """Represents an instance of a bidirectional component."""

    DESCRIPTOR: _descriptor.Descriptor

    ID_FIELD_NUMBER: _builtins.int
    COMPONENT_NAME_FIELD_NUMBER: _builtins.int
    JS_CONTENT_FIELD_NUMBER: _builtins.int
    JS_SOURCE_PATH_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    HTML_CONTENT_FIELD_NUMBER: _builtins.int
    CSS_CONTENT_FIELD_NUMBER: _builtins.int
    CSS_SOURCE_PATH_FIELD_NUMBER: _builtins.int
    ISOLATE_STYLES_FIELD_NUMBER: _builtins.int
    JSON_FIELD_NUMBER: _builtins.int
    ARROW_DATA_FIELD_NUMBER: _builtins.int
    BYTES_FIELD_NUMBER: _builtins.int
    MIXED_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    """The ID of the widget instance."""
    component_name: _builtins.str
    """The registered name of the component."""
    js_content: _builtins.str
    """The JavaScript content provided by the developer."""
    js_source_path: _builtins.str
    """The source path for the JavaScript content."""
    form_id: _builtins.str
    """The ID of the form this component belongs to, if any."""
    html_content: _builtins.str
    """The HTML content provided by the developer."""
    css_content: _builtins.str
    """The CSS content provided by the developer."""
    css_source_path: _builtins.str
    """The source path for the CSS content."""
    isolate_styles: _builtins.bool
    """Whether to isolate styles from the parent."""
    json: _builtins.str
    """JSON-serialized value for generic data payloads."""
    bytes: _builtins.bytes
    """Arbitrary bytes payload."""
    @_builtins.property
    def arrow_data(self) -> _ArrowData_pb2.ArrowData:
        """Data-only Apache Arrow buffer for dataframe-like payloads."""

    @_builtins.property
    def mixed(self) -> Global___MixedData:
        """Mixed data with JSON structure + embedded Arrow references"""

    def __init__(
        self,
        *,
        id: _builtins.str = ...,
        component_name: _builtins.str = ...,
        js_content: _builtins.str | None = ...,
        js_source_path: _builtins.str | None = ...,
        form_id: _builtins.str = ...,
        html_content: _builtins.str | None = ...,
        css_content: _builtins.str | None = ...,
        css_source_path: _builtins.str | None = ...,
        isolate_styles: _builtins.bool = ...,
        json: _builtins.str = ...,
        arrow_data: _ArrowData_pb2.ArrowData | None = ...,
        bytes: _builtins.bytes = ...,
        mixed: Global___MixedData | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_css_content", b"_css_content", "_css_source_path", b"_css_source_path", "_html_content", b"_html_content", "_js_content", b"_js_content", "_js_source_path", b"_js_source_path", "arrow_data", b"arrow_data", "bytes", b"bytes", "css_content", b"css_content", "css_source_path", b"css_source_path", "data", b"data", "html_content", b"html_content", "js_content", b"js_content", "js_source_path", b"js_source_path", "json", b"json", "mixed", b"mixed"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_css_content", b"_css_content", "_css_source_path", b"_css_source_path", "_html_content", b"_html_content", "_js_content", b"_js_content", "_js_source_path", b"_js_source_path", "arrow_data", b"arrow_data", "bytes", b"bytes", "component_name", b"component_name", "css_content", b"css_content", "css_source_path", b"css_source_path", "data", b"data", "form_id", b"form_id", "html_content", b"html_content", "id", b"id", "isolate_styles", b"isolate_styles", "js_content", b"js_content", "js_source_path", b"js_source_path", "json", b"json", "mixed", b"mixed"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__css_content: _TypeAlias = _typing.Literal["css_content"]  # noqa: Y015
    _WhichOneofArgType__css_content: _TypeAlias = _typing.Literal["_css_content", b"_css_content"]  # noqa: Y015
    _WhichOneofReturnType__css_source_path: _TypeAlias = _typing.Literal["css_source_path"]  # noqa: Y015
    _WhichOneofArgType__css_source_path: _TypeAlias = _typing.Literal["_css_source_path", b"_css_source_path"]  # noqa: Y015
    _WhichOneofReturnType__html_content: _TypeAlias = _typing.Literal["html_content"]  # noqa: Y015
    _WhichOneofArgType__html_content: _TypeAlias = _typing.Literal["_html_content", b"_html_content"]  # noqa: Y015
    _WhichOneofReturnType__js_content: _TypeAlias = _typing.Literal["js_content"]  # noqa: Y015
    _WhichOneofArgType__js_content: _TypeAlias = _typing.Literal["_js_content", b"_js_content"]  # noqa: Y015
    _WhichOneofReturnType__js_source_path: _TypeAlias = _typing.Literal["js_source_path"]  # noqa: Y015
    _WhichOneofArgType__js_source_path: _TypeAlias = _typing.Literal["_js_source_path", b"_js_source_path"]  # noqa: Y015
    _WhichOneofReturnType_data: _TypeAlias = _typing.Literal["json", "arrow_data", "bytes", "mixed"]  # noqa: Y015
    _WhichOneofArgType_data: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__css_content) -> _WhichOneofReturnType__css_content | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__css_source_path) -> _WhichOneofReturnType__css_source_path | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__html_content) -> _WhichOneofReturnType__html_content | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__js_content) -> _WhichOneofReturnType__js_content | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__js_source_path) -> _WhichOneofReturnType__js_source_path | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType_data) -> _WhichOneofReturnType_data | None: ...

Global___BidiComponent: _TypeAlias = BidiComponent  # noqa: Y015



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/ArrowNamedDataSet.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from streamlit.proto import ArrowData_pb2 as streamlit_dot_proto_dot_ArrowData__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'streamlit/proto/ArrowNamedDataSet.proto\x1a\x1fstreamlit/proto/ArrowData.proto\"M\n\x11\x41rrowNamedDataSet\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08has_name\x18\x03 \x01(\x08\x12\x18\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\n.ArrowDatab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.ArrowNamedDataSet_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ARROWNAMEDDATASET']._serialized_start=76
  _globals['_ARROWNAMEDDATASET']._serialized_end=153
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from streamlit.proto import ButtonLikeIconPosition_pb2 as _ButtonLikeIconPosition_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class PageLink(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PAGE_FIELD_NUMBER: _builtins.int
    LABEL_FIELD_NUMBER: _builtins.int
    ICON_FIELD_NUMBER: _builtins.int
    PAGE_SCRIPT_HASH_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    USE_CONTAINER_WIDTH_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    EXTERNAL_FIELD_NUMBER: _builtins.int
    QUERY_STRING_FIELD_NUMBER: _builtins.int
    ICON_POSITION_FIELD_NUMBER: _builtins.int
    page: _builtins.str
    label: _builtins.str
    icon: _builtins.str
    page_script_hash: _builtins.str
    help: _builtins.str
    use_container_width: _builtins.bool
    disabled: _builtins.bool
    external: _builtins.bool
    query_string: _builtins.str
    icon_position: _ButtonLikeIconPosition_pb2.ButtonLikeIconPosition.ValueType
    def __init__(
        self,
        *,
        page: _builtins.str = ...,
        label: _builtins.str = ...,
        icon: _builtins.str = ...,
        page_script_hash: _builtins.str = ...,
        help: _builtins.str = ...,
        use_container_width: _builtins.bool | None = ...,
        disabled: _builtins.bool = ...,
        external: _builtins.bool = ...,
        query_string: _builtins.str = ...,
        icon_position: _ButtonLikeIconPosition_pb2.ButtonLikeIconPosition.ValueType = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_use_container_width", b"_use_container_width", "use_container_width", b"use_container_width"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_use_container_width", b"_use_container_width", "disabled", b"disabled", "external", b"external", "help", b"help", "icon", b"icon", "icon_position", b"icon_position", "label", b"label", "page", b"page", "page_script_hash", b"page_script_hash", "query_string", b"query_string", "use_container_width", b"use_container_width"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__use_container_width: _TypeAlias = _typing.Literal["use_container_width"]  # noqa: Y015
    _WhichOneofArgType__use_container_width: _TypeAlias = _typing.Literal["_use_container_width", b"_use_container_width"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__use_container_width) -> _WhichOneofReturnType__use_container_width | None: ...

Global___PageLink: _TypeAlias = PageLink  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Markdown(_message.Message):
    """Formatted text"""

    DESCRIPTOR: _descriptor.Descriptor

    class _Type:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _TypeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Markdown._Type.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        UNSPECIFIED: Markdown._Type.ValueType  # 0
        """This is recommended to be reserved for proto files backwards compatibility reasons."""
        NATIVE: Markdown._Type.ValueType  # 1
        CAPTION: Markdown._Type.ValueType  # 2
        CODE: Markdown._Type.ValueType  # 3
        LATEX: Markdown._Type.ValueType  # 4
        DIVIDER: Markdown._Type.ValueType  # 5

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    UNSPECIFIED: Markdown.Type.ValueType  # 0
    """This is recommended to be reserved for proto files backwards compatibility reasons."""
    NATIVE: Markdown.Type.ValueType  # 1
    CAPTION: Markdown.Type.ValueType  # 2
    CODE: Markdown.Type.ValueType  # 3
    LATEX: Markdown.Type.ValueType  # 4
    DIVIDER: Markdown.Type.ValueType  # 5

    BODY_FIELD_NUMBER: _builtins.int
    ALLOW_HTML_FIELD_NUMBER: _builtins.int
    ELEMENT_TYPE_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    UNTERMINATED_PARSING_FIELD_NUMBER: _builtins.int
    body: _builtins.str
    """Content to display."""
    allow_html: _builtins.bool
    element_type: Global___Markdown.Type.ValueType
    help: _builtins.str
    unterminated_parsing: _builtins.bool
    """Enables frontend unterminated markdown completion (via remend) during streaming."""
    def __init__(
        self,
        *,
        body: _builtins.str = ...,
        allow_html: _builtins.bool = ...,
        element_type: Global___Markdown.Type.ValueType = ...,
        help: _builtins.str = ...,
        unterminated_parsing: _builtins.bool = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["allow_html", b"allow_html", "body", b"body", "element_type", b"element_type", "help", b"help", "unterminated_parsing", b"unterminated_parsing"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Markdown: _TypeAlias = Markdown  # noqa: Y015



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Serve static assets for Custom Components v2.

This module defines a Tornado ``RequestHandler`` that serves static files for
Custom Components v2 from their registered component directories. Requests are
resolved safely within the component's root to avoid directory traversal and are
served with appropriate content type and cache headers. CORS headers are applied
based on the server configuration.
"""

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Final, cast

import tornado.web

import streamlit.web.server.routes
from streamlit.logger import get_logger
from streamlit.web.server.component_file_utils import (
    build_safe_abspath,
    guess_content_type,
)

if TYPE_CHECKING:
    from streamlit.components.v2.component_manager import BidiComponentManager

_LOGGER: Final = get_logger(__name__)


class BidiComponentRequestHandler(tornado.web.RequestHandler):
    """Request handler for serving Custom Components v2 static assets.

    The handler resolves a requested path to a registered component's asset
    within its component root, writes the file contents to the response, and
    sets appropriate ``Content-Type`` and cache headers. If the component or
    asset cannot be found, a suitable HTTP status is returned.
    """

    def initialize(self, component_manager: BidiComponentManager) -> None:
        """Initialize the handler with the given component manager.

        Parameters
        ----------
        component_manager : BidiComponentManager
            Manager used to look up registered components and their root paths.
        """
        self._component_manager = component_manager

    def get(self, path: str) -> None:
        """Serve a component asset for the given URL path.

        The first path segment is interpreted as the component name. The rest
        of the path is resolved to a file within that component's root
        directory. If the file exists and is readable, its bytes are written to
        the response and the ``Content-Type`` header is set based on the file
        type.

        Parameters
        ----------
        path : str
            Request path in the form ``"<component_name>/<relative_file>"``.

        Notes
        -----
        This method writes directly to the response and sets appropriate HTTP
        status codes on error (``404`` for missing components/files, ``400`` for
        unsafe paths).
        """
        parts = path.split("/")
        component_name = parts[0]
        component_def = self._component_manager.get(component_name)
        if component_def is None:
            self.write("not found")
            self.set_status(404)
            return

        # Get the component path from the component manager
        component_path = self._component_manager.get_component_path(component_name)
        if component_path is None:
            self.write("not found")
            self.set_status(404)
            return

        # Build a safe absolute path within the component root
        filename = "/".join(parts[1:])
        # If no file segment is provided (e.g. only component name or trailing slash),
        # treat as not found rather than attempting to open a directory.
        if not filename or filename.endswith("/"):
            self.write("not found")
            self.set_status(404)
            return
        abspath = build_safe_abspath(component_path, filename)
        if abspath is None:
            self.write("Bad Request")
            self.set_status(400)
            return

        # If the resolved path is a directory, return 404 not found.
        if os.path.isdir(abspath):
            self.write("not found")
            self.set_status(404)
            return

        try:
            with open(abspath, "rb") as file:
                contents = file.read()
        except OSError:
            sanitized_abspath = abspath.replace("\n", "").replace("\r", "")
            _LOGGER.exception(
                "BidiComponentRequestHandler: GET %s read error", sanitized_abspath
            )
            self.write("read error")
            self.set_status(404)
            return

        self.write(contents)
        self.set_header("Content-Type", guess_content_type(abspath))

        self.set_extra_headers(path)

    def set_extra_headers(self, path: str) -> None:
        """Disable cache for HTML files.

        We assume other assets like JS and CSS are suffixed with their hash, so
        they can be cached indefinitely.
        """
        if path.endswith(".html"):
            self.set_header("Cache-Control", "no-cache")
        else:
            self.set_header("Cache-Control", "public")

    def set_default_headers(self) -> None:
        """Set default CORS headers based on server configuration.

        If cross-origin requests are fully allowed, ``Access-Control-Allow-
        Origin`` is set to ``"*"``. Otherwise, if the request ``Origin`` header
        is an allowed origin, the header is echoed back.
        """
        if streamlit.web.server.routes.allow_all_cross_origin_requests():
            self.set_header("Access-Control-Allow-Origin", "*")
        elif streamlit.web.server.routes.is_allowed_origin(
            origin := self.request.headers.get("Origin")
        ):
            self.set_header("Access-Control-Allow-Origin", cast("str", origin))

    def options(self) -> None:
        """Handle preflight CORS requests.

        Responds with HTTP ``204 No Content`` to indicate that the actual
        request can proceed.
        """
        self.set_status(204)
        self.finish()

    @staticmethod
    def get_url(file_id: str) -> str:
        """Return the URL for a component asset identified by ``file_id``.

        Parameters
        ----------
        file_id : str
            Component file identifier (typically a relative path or hashed
            filename).

        Returns
        -------
        str
            Relative URL path for the resource, to be joined with the server
            base URL.

        Examples
        --------
        >>> BidiComponentRequestHandler.get_url("my_component/main.js")
        '_stcore/bidi-components/my_component/main.js'
        """
        return f"_stcore/bidi-components/{file_id}"



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import hmac
import json
from typing import TYPE_CHECKING, Any, Final
from urllib.parse import urlparse

import tornado.websocket
from tornado.escape import utf8
from tornado.websocket import WebSocketHandler

from streamlit import config
from streamlit.auth_util import get_cookie_with_chunks, get_expose_tokens_config
from streamlit.logger import get_logger
from streamlit.proto.BackMsg_pb2 import BackMsg
from streamlit.runtime import Runtime, SessionClient, SessionClientDisconnectedError
from streamlit.runtime.runtime_util import serialize_forward_msg
from streamlit.runtime.session_manager import ClientContext
from streamlit.web.server.server_util import (
    AUTH_COOKIE_NAME,
    TOKENS_COOKIE_NAME,
    is_url_from_allowed_origins,
    is_xsrf_enabled,
)

if TYPE_CHECKING:
    from collections.abc import Awaitable, Iterable, Mapping

    from tornado.httputil import HTTPServerRequest

    from streamlit.proto.ForwardMsg_pb2 import ForwardMsg

_LOGGER: Final = get_logger(__name__)


class TornadoClientContext(ClientContext):
    """Tornado-specific implementation of ClientContext.

    Captures headers, cookies, and client info from the initial WebSocket handshake.
    Values are cached at construction time since they represent the initial request
    context and should not change during the connection lifetime.
    """

    def __init__(self, tornado_request: HTTPServerRequest) -> None:
        self._headers: list[tuple[str, str]] = list(tornado_request.headers.get_all())
        self._cookies: dict[str, str] = {
            k: m.value for k, m in tornado_request.cookies.items()
        }
        self._remote_ip: str | None = tornado_request.remote_ip

    @property
    def headers(self) -> Iterable[tuple[str, str]]:
        """All headers as (name, value) tuples."""
        return self._headers

    @property
    def cookies(self) -> Mapping[str, str]:
        """Cookies as a name-to-value mapping."""
        return self._cookies

    @property
    def remote_ip(self) -> str | None:
        """The client's remote IP address."""
        return self._remote_ip


class BrowserWebSocketHandler(WebSocketHandler, SessionClient):
    """Handles a WebSocket connection from the browser."""

    def initialize(self, runtime: Runtime) -> None:
        self._runtime = runtime
        self._session_id: str | None = None
        # The XSRF cookie is normally set when xsrf_form_html is used, but in a
        # pure-Javascript application that does not use any regular forms we just
        # need to read the self.xsrf_token manually to set the cookie as a side
        # effect. See https://www.tornadoweb.org/en/stable/guide/security.html#cross-site-request-forgery-protection
        # for more details.
        if is_xsrf_enabled():
            _ = self.xsrf_token

        # Do this once instead of on every open
        self.expose_tokens = get_expose_tokens_config()

    def get_signed_cookie(
        self,
        name: str,
        value: str | None = None,
        max_age_days: float = 31,
        min_version: int | None = None,
    ) -> bytes | None:
        """Get a signed cookie from the request, reconstructing from chunks if needed.

        Added for compatibility with Tornado < 6.3.0. Also handles chunked cookies
        automatically.

        See release notes: https://www.tornadoweb.org/en/stable/releases/v6.3.0.html#deprecation-notices
        """

        def get_single_cookie(cookie_name: str) -> bytes | None:
            try:
                return super(BrowserWebSocketHandler, self).get_signed_cookie(
                    cookie_name, value, max_age_days, min_version
                )
            except AttributeError:
                return super(BrowserWebSocketHandler, self).get_secure_cookie(
                    cookie_name, value, max_age_days, min_version
                )

        return get_cookie_with_chunks(get_single_cookie, name)

    def check_origin(self, origin: str) -> bool:
        """Set up CORS."""
        return super().check_origin(origin) or is_url_from_allowed_origins(origin)

    def _validate_xsrf_token(self, supplied_token: str) -> bool:
        """Inspired by tornado.web.RequestHandler.check_xsrf_cookie method,
        to check the XSRF token passed in Websocket connection header.
        """
        _, token, _ = self._decode_xsrf_token(supplied_token)
        _, expected_token, _ = self._get_raw_xsrf_token()

        decoded_token = utf8(token)
        decoded_expected_token = utf8(expected_token)

        if not decoded_token or not decoded_expected_token:
            return False
        return hmac.compare_digest(decoded_token, decoded_expected_token)

    def _parse_user_cookie(self, raw_cookie_value: bytes) -> dict[str, Any]:
        """Process the user cookie and extract the user info after
        validating the origin. Origin is validated for security reasons.
        """
        cookie_value = json.loads(raw_cookie_value)
        user_info = {}

        cookie_value_origin = cookie_value.get("origin", None)
        parsed_origin_from_header = urlparse(self.request.headers["Origin"])
        expected_origin_value = (
            parsed_origin_from_header.scheme + "://" + parsed_origin_from_header.netloc
        )
        if cookie_value_origin == expected_origin_value:
            user_info["is_logged_in"] = cookie_value.get("is_logged_in", False)
            del cookie_value["origin"]
            del cookie_value["is_logged_in"]
            user_info.update(cookie_value)

        else:
            _LOGGER.error(
                "Origin mismatch, the origin of websocket request is not the "
                "same origin of redirect_uri in secrets.toml",
            )

        return user_info

    def write_forward_msg(self, msg: ForwardMsg) -> None:
        """Send a ForwardMsg to the browser."""
        try:
            self.write_message(serialize_forward_msg(msg), binary=True)
        except tornado.websocket.WebSocketClosedError as e:
            raise SessionClientDisconnectedError from e

    @property
    def client_context(self) -> ClientContext:
        """Return the client's connection context.

        The context is cached on first access to avoid repeatedly
        constructing a new TornadoClientContext instance.
        """
        context = getattr(self, "_client_context", None)
        if context is None:
            context = TornadoClientContext(self.request)
            self._client_context = context
        return context

    def select_subprotocol(self, subprotocols: list[str]) -> str | None:
        """Return the first subprotocol in the given list.

        This method is used by Tornado to select a protocol when the
        Sec-WebSocket-Protocol header is set in an HTTP Upgrade request.

        NOTE: We repurpose the Sec-WebSocket-Protocol header here in a slightly
        unfortunate (but necessary) way. The browser WebSocket API doesn't allow us to
        set arbitrary HTTP headers, and this header is the only one where we have the
        ability to set it to arbitrary values, so we use it to pass tokens (in this
        case, the previous session ID to allow us to reconnect to it) from client to
        server as the *third* value in the list.

        The reason why the auth token is set as the third value is that:
          - when Sec-WebSocket-Protocol is set, many clients expect the server to
            respond with a selected subprotocol to use. We don't want that reply to be
            the session token, so we by convention have the client always set the first
            protocol to "streamlit" and select that.
          - the second protocol in the list is reserved in some deployment environments
            for an auth token that we currently don't use
        """
        if subprotocols:
            return subprotocols[0]

        return None

    def open(self, *args: Any, **kwargs: Any) -> Awaitable[None] | None:
        user_info: dict[str, dict[str, str] | str | bool | None] = {}

        existing_session_id = None
        try:
            ws_protocols = [
                p.strip()
                for p in self.request.headers["Sec-Websocket-Protocol"].split(",")
            ]

            raw_cookie_value = self.get_signed_cookie(AUTH_COOKIE_NAME)
            if is_xsrf_enabled() and raw_cookie_value:
                csrf_protocol_value = ws_protocols[1]

                if self._validate_xsrf_token(csrf_protocol_value):
                    user_info.update(self._parse_user_cookie(raw_cookie_value))

                    # Also read in tokens if token cookie exists
                    raw_token_cookie_value = self.get_signed_cookie(TOKENS_COOKIE_NAME)
                    if raw_token_cookie_value:
                        all_tokens = json.loads(raw_token_cookie_value)

                        # Filter tokens based on expose_tokens configuration
                        filtered_tokens: dict[str, str] = {}
                        for token_type in self.expose_tokens:
                            token_key = f"{token_type}_token"
                            if token_key in all_tokens:
                                filtered_tokens[token_type] = all_tokens[token_key]

                        user_info["tokens"] = filtered_tokens

            if len(ws_protocols) >= 3:
                # See the NOTE in the docstring of the `select_subprotocol` method above
                # for a detailed explanation of why this is done.
                existing_session_id = ws_protocols[2]
        except (KeyError, json.JSONDecodeError):
            # Just let existing_session_id=None if we run into any error while trying to
            # extract it from the Sec-Websocket-Protocol header or parsing cookie JSON.
            pass

        # Map in any user-configured headers. Note that these override anything coming
        # from the auth cookie.
        mapping_config = config.get_option("server.trustedUserHeaders")
        for header_name, user_info_key in mapping_config.items():
            header_values = self.request.headers.get_list(header_name)
            if header_values:
                # If there's at least one value, use the first value.
                # NOTE: Tornado doesn't document the order of these values, so it's
                # possible this won't be the first value that was received by the
                # server.
                user_info[user_info_key] = header_values[0]
            else:
                # Default to explicit None.
                user_info[user_info_key] = None

        self._session_id = self._runtime.connect_session(
            client=self,
            user_info=user_info,
            existing_session_id=existing_session_id,
        )
        return None

    def on_close(self) -> None:
        if not self._session_id:
            return
        self._runtime.disconnect_session(self._session_id)
        self._session_id = None

    def get_compression_options(self) -> dict[Any, Any] | None:
        """Enable WebSocket compression.

        Returning an empty dict enables websocket compression. Returning
        None disables it.

        (See the docstring in the parent class.)
        """
        if config.get_option("server.enableWebsocketCompression"):
            return {}
        return None

    def on_message(self, message: str | bytes) -> None:
        if not self._session_id:
            return

        try:
            if isinstance(message, str):
                # Sanity check. (The frontend should only be sending us bytes;
                # Protobuf.ParseFromString does not accept str input.)
                raise TypeError(  # noqa: TRY301
                    "WebSocket received an unexpected `str` message. "
                    "(We expect `bytes` only.)"
                )

            msg = BackMsg()
            msg.ParseFromString(message)
            _LOGGER.debug("Received the following back message:\n%s", msg)

        except Exception as ex:
            _LOGGER.exception("Error deserializing back message")
            self._runtime.handle_backmsg_deserialization_exception(self._session_id, ex)
            return

        # "debug_disconnect_websocket" and "debug_shutdown_runtime" are special
        # developmentMode-only messages used in e2e tests to test reconnect handling and
        # disabling widgets.
        if msg.WhichOneof("type") == "debug_disconnect_websocket":
            if config.get_option("global.developmentMode") or config.get_option(
                "global.e2eTest"
            ):
                self.close()
            else:
                _LOGGER.warning(
                    "Client tried to disconnect websocket when not in development mode or e2e testing."
                )
        elif msg.WhichOneof("type") == "debug_shutdown_runtime":
            if config.get_option("global.developmentMode") or config.get_option(
                "global.e2eTest"
            ):
                self._runtime.stop()
            else:
                _LOGGER.warning(
                    "Client tried to shut down runtime when not in development mode or e2e testing."
                )
        else:
            # AppSession handles all other BackMsg types.
            self._runtime.handle_backmsg(self._session_id, msg)
        return



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class PageConfig(_message.Message):
    """Properties of the page that the app creator may configure."""

    DESCRIPTOR: _descriptor.Descriptor

    class _Layout:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _LayoutEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[PageConfig._Layout.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        CENTERED: PageConfig._Layout.ValueType  # 0
        """Render page elements in a centered block, roughly 730px wide."""
        WIDE: PageConfig._Layout.ValueType  # 1
        """Use the full browser width to render elements."""
        LAYOUT_UNSET: PageConfig._Layout.ValueType  # 2
        """Unset distinguishes that config was not provided - on initial call, will
        set to centered as default. Otherwise, inherit value from previous call
        """

    class Layout(_Layout, metaclass=_LayoutEnumTypeWrapper): ...
    CENTERED: PageConfig.Layout.ValueType  # 0
    """Render page elements in a centered block, roughly 730px wide."""
    WIDE: PageConfig.Layout.ValueType  # 1
    """Use the full browser width to render elements."""
    LAYOUT_UNSET: PageConfig.Layout.ValueType  # 2
    """Unset distinguishes that config was not provided - on initial call, will
    set to centered as default. Otherwise, inherit value from previous call
    """

    class _SidebarState:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _SidebarStateEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[PageConfig._SidebarState.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        AUTO: PageConfig._SidebarState.ValueType  # 0
        """Expand the sidebar if page width exceeds ~1000px; otherwise, collapse it"""
        EXPANDED: PageConfig._SidebarState.ValueType  # 1
        """Force the sidebar to be shown."""
        COLLAPSED: PageConfig._SidebarState.ValueType  # 2
        """Force the sidebar to be hidden."""
        SIDEBAR_UNSET: PageConfig._SidebarState.ValueType  # 3
        """Unset distinguishes that config was not provided - on initial call, will
        set to auto as default. Otherwise, inherit value from previous call
        """

    class SidebarState(_SidebarState, metaclass=_SidebarStateEnumTypeWrapper): ...
    AUTO: PageConfig.SidebarState.ValueType  # 0
    """Expand the sidebar if page width exceeds ~1000px; otherwise, collapse it"""
    EXPANDED: PageConfig.SidebarState.ValueType  # 1
    """Force the sidebar to be shown."""
    COLLAPSED: PageConfig.SidebarState.ValueType  # 2
    """Force the sidebar to be hidden."""
    SIDEBAR_UNSET: PageConfig.SidebarState.ValueType  # 3
    """Unset distinguishes that config was not provided - on initial call, will
    set to auto as default. Otherwise, inherit value from previous call
    """

    @_typing.final
    class MenuItems(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        GET_HELP_URL_FIELD_NUMBER: _builtins.int
        HIDE_GET_HELP_FIELD_NUMBER: _builtins.int
        REPORT_A_BUG_URL_FIELD_NUMBER: _builtins.int
        HIDE_REPORT_A_BUG_FIELD_NUMBER: _builtins.int
        ABOUT_SECTION_MD_FIELD_NUMBER: _builtins.int
        CLEAR_ABOUT_MD_FIELD_NUMBER: _builtins.int
        get_help_url: _builtins.str
        hide_get_help: _builtins.bool
        report_a_bug_url: _builtins.str
        hide_report_a_bug: _builtins.bool
        about_section_md: _builtins.str
        clear_about_md: _builtins.bool
        """For multiple calls to set_page_config, clears set about markdown"""
        def __init__(
            self,
            *,
            get_help_url: _builtins.str = ...,
            hide_get_help: _builtins.bool = ...,
            report_a_bug_url: _builtins.str = ...,
            hide_report_a_bug: _builtins.bool = ...,
            about_section_md: _builtins.str = ...,
            clear_about_md: _builtins.bool = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["about_section_md", b"about_section_md", "clear_about_md", b"clear_about_md", "get_help_url", b"get_help_url", "hide_get_help", b"hide_get_help", "hide_report_a_bug", b"hide_report_a_bug", "report_a_bug_url", b"report_a_bug_url"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    @_typing.final
    class SidebarWidthConfig(_message.Message):
        """Configuration for the initial sidebar width.
        Uses a oneof pattern to allow for future extensions (e.g., rem units).
        """

        DESCRIPTOR: _descriptor.Descriptor

        PIXEL_WIDTH_FIELD_NUMBER: _builtins.int
        pixel_width: _builtins.int
        def __init__(
            self,
            *,
            pixel_width: _builtins.int = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["pixel_width", b"pixel_width", "width_spec", b"width_spec"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["pixel_width", b"pixel_width", "width_spec", b"width_spec"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType_width_spec: _TypeAlias = _typing.Literal["pixel_width"]  # noqa: Y015
        _WhichOneofArgType_width_spec: _TypeAlias = _typing.Literal["width_spec", b"width_spec"]  # noqa: Y015
        def WhichOneof(self, oneof_group: _WhichOneofArgType_width_spec) -> _WhichOneofReturnType_width_spec | None: ...

    TITLE_FIELD_NUMBER: _builtins.int
    FAVICON_FIELD_NUMBER: _builtins.int
    LAYOUT_FIELD_NUMBER: _builtins.int
    INITIAL_SIDEBAR_STATE_FIELD_NUMBER: _builtins.int
    MENU_ITEMS_FIELD_NUMBER: _builtins.int
    INITIAL_SIDEBAR_WIDTH_FIELD_NUMBER: _builtins.int
    title: _builtins.str
    favicon: _builtins.str
    layout: Global___PageConfig.Layout.ValueType
    initial_sidebar_state: Global___PageConfig.SidebarState.ValueType
    @_builtins.property
    def menu_items(self) -> Global___PageConfig.MenuItems: ...
    @_builtins.property
    def initial_sidebar_width(self) -> Global___PageConfig.SidebarWidthConfig: ...
    def __init__(
        self,
        *,
        title: _builtins.str = ...,
        favicon: _builtins.str = ...,
        layout: Global___PageConfig.Layout.ValueType = ...,
        initial_sidebar_state: Global___PageConfig.SidebarState.ValueType = ...,
        menu_items: Global___PageConfig.MenuItems | None = ...,
        initial_sidebar_width: Global___PageConfig.SidebarWidthConfig | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_initial_sidebar_width", b"_initial_sidebar_width", "initial_sidebar_width", b"initial_sidebar_width", "menu_items", b"menu_items"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_initial_sidebar_width", b"_initial_sidebar_width", "favicon", b"favicon", "initial_sidebar_state", b"initial_sidebar_state", "initial_sidebar_width", b"initial_sidebar_width", "layout", b"layout", "menu_items", b"menu_items", "title", b"title"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__initial_sidebar_width: _TypeAlias = _typing.Literal["initial_sidebar_width"]  # noqa: Y015
    _WhichOneofArgType__initial_sidebar_width: _TypeAlias = _typing.Literal["_initial_sidebar_width", b"_initial_sidebar_width"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__initial_sidebar_width) -> _WhichOneofReturnType__initial_sidebar_width | None: ...

Global___PageConfig: _TypeAlias = PageConfig  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Logo(_message.Message):
    """Handle the logo for an app"""

    DESCRIPTOR: _descriptor.Descriptor

    class _ImageType:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _ImageTypeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Logo._ImageType.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        IMAGE: Logo._ImageType.ValueType  # 0
        EMOJI: Logo._ImageType.ValueType  # 1
        ICON: Logo._ImageType.ValueType  # 2

    class ImageType(_ImageType, metaclass=_ImageTypeEnumTypeWrapper):
        """Type of the logo image (image file, emoji, or material icon)"""

    IMAGE: Logo.ImageType.ValueType  # 0
    EMOJI: Logo.ImageType.ValueType  # 1
    ICON: Logo.ImageType.ValueType  # 2

    IMAGE_FIELD_NUMBER: _builtins.int
    LINK_FIELD_NUMBER: _builtins.int
    ICON_IMAGE_FIELD_NUMBER: _builtins.int
    SIZE_FIELD_NUMBER: _builtins.int
    IMAGE_TYPE_FIELD_NUMBER: _builtins.int
    ICON_IMAGE_TYPE_FIELD_NUMBER: _builtins.int
    image: _builtins.str
    link: _builtins.str
    icon_image: _builtins.str
    size: _builtins.str
    image_type: Global___Logo.ImageType.ValueType
    icon_image_type: Global___Logo.ImageType.ValueType
    def __init__(
        self,
        *,
        image: _builtins.str = ...,
        link: _builtins.str = ...,
        icon_image: _builtins.str = ...,
        size: _builtins.str = ...,
        image_type: Global___Logo.ImageType.ValueType = ...,
        icon_image_type: Global___Logo.ImageType.ValueType = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["icon_image", b"icon_image", "icon_image_type", b"icon_image_type", "image", b"image", "image_type", b"image_type", "link", b"link", "size", b"size"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Logo: _TypeAlias = Logo  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Toast(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    BODY_FIELD_NUMBER: _builtins.int
    ICON_FIELD_NUMBER: _builtins.int
    DURATION_FIELD_NUMBER: _builtins.int
    body: _builtins.str
    """Display message"""
    icon: _builtins.str
    """Emoji"""
    duration: _builtins.int
    """The time to display the toast message in seconds.
    If not set, the toast will be displayed for 4 seconds. A value of 0
    indicates that the toast should not be automatically dismissed.
    """
    def __init__(
        self,
        *,
        body: _builtins.str = ...,
        icon: _builtins.str = ...,
        duration: _builtins.int | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_duration", b"_duration", "duration", b"duration"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_duration", b"_duration", "body", b"body", "duration", b"duration", "icon", b"icon"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__duration: _TypeAlias = _typing.Literal["duration"]  # noqa: Y015
    _WhichOneofArgType__duration: _TypeAlias = _typing.Literal["_duration", b"_duration"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__duration) -> _WhichOneofReturnType__duration | None: ...

Global___Toast: _TypeAlias = Toast  # noqa: Y015



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/MenuButton.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n streamlit/proto/MenuButton.proto\"t\n\nMenuButton\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05label\x18\x02 \x01(\t\x12\x0f\n\x07options\x18\x03 \x03(\t\x12\x0c\n\x04help\x18\x04 \x01(\t\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\x10\n\x08\x64isabled\x18\x06 \x01(\x08\x12\x0c\n\x04icon\x18\x07 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.MenuButton_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MENUBUTTON']._serialized_start=36
  _globals['_MENUBUTTON']._serialized_end=152
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/Exception.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from streamlit.proto import WidthConfig_pb2 as streamlit_dot_proto_dot_WidthConfig__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fstreamlit/proto/Exception.proto\x1a!streamlit/proto/WidthConfig.proto\"\x9e\x01\n\tException\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x1b\n\x13message_is_markdown\x18\x04 \x01(\x08\x12\x13\n\x0bstack_trace\x18\x03 \x03(\t\x12\x12\n\nis_warning\x18\x05 \x01(\x08\x12,\n\x0cwidth_config\x18\x06 \x01(\x0b\x32\x16.streamlit.WidthConfigb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.Exception_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EXCEPTION']._serialized_start=71
  _globals['_EXCEPTION']._serialized_end=229
# @@protoc_insertion_point(module_scope)



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from typing import TYPE_CHECKING, cast

import tornado.web

from streamlit.runtime.stats import metric_type_string_to_proto
from streamlit.web.server import allow_all_cross_origin_requests, is_allowed_origin
from streamlit.web.server.server_util import emit_endpoint_deprecation_notice

if TYPE_CHECKING:
    from collections.abc import Mapping, Sequence

    from streamlit.proto.openmetrics_data_model_pb2 import MetricSet as MetricSetProto
    from streamlit.runtime.stats import Stat, StatsManager


class StatsRequestHandler(tornado.web.RequestHandler):
    def initialize(self, stats_manager: StatsManager) -> None:
        self._manager = stats_manager

    def set_default_headers(self) -> None:
        if allow_all_cross_origin_requests():
            self.set_header("Access-Control-Allow-Origin", "*")
        elif is_allowed_origin(origin := self.request.headers.get("Origin")):
            self.set_header("Access-Control-Allow-Origin", cast("str", origin))

    def options(self) -> None:
        """/OPTIONS handler for preflight CORS checks."""
        self.set_status(204)
        self.finish()

    def get(self) -> None:
        if self.request.uri and "_stcore/" not in self.request.uri:
            emit_endpoint_deprecation_notice(self, new_path="/_stcore/metrics")

        # Allow caller to request specific metric families via query parameter.
        # If no families are specified, all metrics are returned.
        # Example: /_stcore/metrics?families=session_events_total&families=active_sessions
        requested_families = self.get_arguments("families")
        stats = self._manager.get_stats(family_names=requested_families or None)
        # If the request asked for protobuf output, we return a serialized
        # protobuf. Else we return text.
        if "application/x-protobuf" in self.request.headers.get_list("Accept"):
            self.write(self._stats_to_proto(stats).SerializeToString())
            self.set_header("Content-Type", "application/x-protobuf")
            self.set_status(200)
        else:
            self.write(self._stats_to_text(stats))
            self.set_header("Content-Type", "application/openmetrics-text")
            self.set_status(200)

    @staticmethod
    def _stats_to_text(stats_by_family: Mapping[str, Sequence[Stat]]) -> str:
        result: list[str] = []

        for stats in stats_by_family.values():
            if not stats:
                continue

            # All of the stats in a family will have the same family_name, type,
            # unit, and help text, so we can just use the first one to construct
            # our OpenMetrics comments.
            first_stat = stats[0]
            result.append(f"# TYPE {first_stat.family_name} {first_stat.type}")
            result.append(f"# UNIT {first_stat.family_name} {first_stat.unit}")
            result.append(f"# HELP {first_stat.help}")
            result.extend(stat.to_metric_str() for stat in stats)

        result.append("# EOF\n")
        return "\n".join(result)

    @staticmethod
    def _stats_to_proto(
        stats_by_family: Mapping[str, Sequence[Stat]],
    ) -> MetricSetProto:
        # Lazy load the import of this proto message for better performance:
        from streamlit.proto.openmetrics_data_model_pb2 import (
            MetricSet as MetricSetProto,
        )

        metric_set = MetricSetProto()

        for stats in stats_by_family.values():
            if not stats:
                continue

            # All of the stats in a family will have the same family_name, type,
            # unit, and help text, so we can just use the first one to fill in
            # these metric_family fields.
            first_stat = stats[0]
            metric_family = metric_set.metric_families.add()
            metric_family.name = first_stat.family_name
            metric_family.type = metric_type_string_to_proto(first_stat.type)
            metric_family.unit = first_stat.unit
            metric_family.help = first_stat.help

            for stat in stats:
                metric_proto = metric_family.metrics.add()
                stat.marshall_metric_proto(metric_proto)

        return metric_set



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from streamlit.proto import WidthConfig_pb2 as _WidthConfig_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Audio(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    URL_FIELD_NUMBER: _builtins.int
    START_TIME_FIELD_NUMBER: _builtins.int
    END_TIME_FIELD_NUMBER: _builtins.int
    LOOP_FIELD_NUMBER: _builtins.int
    AUTOPLAY_FIELD_NUMBER: _builtins.int
    ID_FIELD_NUMBER: _builtins.int
    WIDTH_CONFIG_FIELD_NUMBER: _builtins.int
    url: _builtins.str
    start_time: _builtins.int
    """The currentTime attribute of the HTML <audio> tag's <source> subtag."""
    end_time: _builtins.int
    """The time at which the audio should stop playing. If not specified, plays to the end."""
    loop: _builtins.bool
    """Indicates whether the audio should start over from the beginning once it ends."""
    autoplay: _builtins.bool
    id: _builtins.str
    @_builtins.property
    def width_config(self) -> _WidthConfig_pb2.WidthConfig: ...
    def __init__(
        self,
        *,
        url: _builtins.str = ...,
        start_time: _builtins.int = ...,
        end_time: _builtins.int = ...,
        loop: _builtins.bool = ...,
        autoplay: _builtins.bool = ...,
        id: _builtins.str = ...,
        width_config: _WidthConfig_pb2.WidthConfig | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_width_config", b"_width_config", "width_config", b"width_config"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_width_config", b"_width_config", "autoplay", b"autoplay", "end_time", b"end_time", "id", b"id", "loop", b"loop", "start_time", b"start_time", "url", b"url", "width_config", b"width_config"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__width_config: _TypeAlias = _typing.Literal["width_config"]  # noqa: Y015
    _WhichOneofArgType__width_config: _TypeAlias = _typing.Literal["_width_config", b"_width_config"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__width_config) -> _WhichOneofReturnType__width_config | None: ...

Global___Audio: _TypeAlias = Audio  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import WidthConfig_pb2 as _WidthConfig_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Alert(_message.Message):
    """NOTE: This proto type is used by some external services so needs to remain
    relatively stable. While it isn't entirely set in stone, changing it
    may require a good amount of effort so should be avoided if possible.
    """

    DESCRIPTOR: _descriptor.Descriptor

    class _Format:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _FormatEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Alert._Format.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        UNUSED: Alert._Format.ValueType  # 0
        """Plain, fixed width text."""
        ERROR: Alert._Format.ValueType  # 1
        """Shows an error message."""
        WARNING: Alert._Format.ValueType  # 2
        """Shows a warning message."""
        INFO: Alert._Format.ValueType  # 3
        """Shows an info log."""
        SUCCESS: Alert._Format.ValueType  # 4
        """Shows a success message."""

    class Format(_Format, metaclass=_FormatEnumTypeWrapper):
        """Type of Alert"""

    UNUSED: Alert.Format.ValueType  # 0
    """Plain, fixed width text."""
    ERROR: Alert.Format.ValueType  # 1
    """Shows an error message."""
    WARNING: Alert.Format.ValueType  # 2
    """Shows a warning message."""
    INFO: Alert.Format.ValueType  # 3
    """Shows an info log."""
    SUCCESS: Alert.Format.ValueType  # 4
    """Shows a success message."""

    BODY_FIELD_NUMBER: _builtins.int
    FORMAT_FIELD_NUMBER: _builtins.int
    ICON_FIELD_NUMBER: _builtins.int
    WIDTH_CONFIG_FIELD_NUMBER: _builtins.int
    body: _builtins.str
    """Content to display."""
    format: Global___Alert.Format.ValueType
    icon: _builtins.str
    @_builtins.property
    def width_config(self) -> _WidthConfig_pb2.WidthConfig:
        """Indicates the width setting: "stetch", "content" or a pixel value."""

    def __init__(
        self,
        *,
        body: _builtins.str = ...,
        format: Global___Alert.Format.ValueType = ...,
        icon: _builtins.str = ...,
        width_config: _WidthConfig_pb2.WidthConfig | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["width_config", b"width_config"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["body", b"body", "format", b"format", "icon", b"icon", "width_config", b"width_config"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Alert: _TypeAlias = Alert  # noqa: Y015



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/Code.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1astreamlit/proto/Code.proto\"`\n\x04\x43ode\x12\x11\n\tcode_text\x18\x01 \x01(\t\x12\x10\n\x08language\x18\x02 \x01(\t\x12\x19\n\x11show_line_numbers\x18\x03 \x01(\x08\x12\x12\n\nwrap_lines\x18\x04 \x01(\x08J\x04\x08\x05\x10\x06\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.Code_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CODE']._serialized_start=30
  _globals['_CODE']._serialized_end=126
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from streamlit.proto import Element_pb2 as _Element_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Transient(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ELEMENTS_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def elements(self) -> _containers.RepeatedCompositeFieldContainer[_Element_pb2.Element]:
        """The list of currently active transient elements (e.g. spinners) at one delta path position"""

    def __init__(
        self,
        *,
        elements: _abc.Iterable[_Element_pb2.Element] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["elements", b"elements"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Transient: _TypeAlias = Transient  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from streamlit.proto import LabelVisibility_pb2 as _LabelVisibility_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class CameraInput(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ID_FIELD_NUMBER: _builtins.int
    LABEL_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    LABEL_VISIBILITY_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    label: _builtins.str
    help: _builtins.str
    form_id: _builtins.str
    disabled: _builtins.bool
    @_builtins.property
    def label_visibility(self) -> _LabelVisibility_pb2.LabelVisibility: ...
    def __init__(
        self,
        *,
        id: _builtins.str = ...,
        label: _builtins.str = ...,
        help: _builtins.str = ...,
        form_id: _builtins.str = ...,
        disabled: _builtins.bool = ...,
        label_visibility: _LabelVisibility_pb2.LabelVisibility | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["label_visibility", b"label_visibility"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["disabled", b"disabled", "form_id", b"form_id", "help", b"help", "id", b"id", "label", b"label", "label_visibility", b"label_visibility"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___CameraInput: _TypeAlias = CameraInput  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
The OpenMetrics protobuf schema which defines the protobuf wire format.
Ensure to interpret "required" as semantically required for a valid message.
All string fields MUST be UTF-8 encoded strings.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

class _MetricType:
    ValueType = _typing.NewType("ValueType", _builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _MetricTypeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[_MetricType.ValueType], _builtins.type):
    DESCRIPTOR: _descriptor.EnumDescriptor
    UNKNOWN: _MetricType.ValueType  # 0
    """Unknown must use unknown MetricPoint values."""
    GAUGE: _MetricType.ValueType  # 1
    """Gauge must use gauge MetricPoint values."""
    COUNTER: _MetricType.ValueType  # 2
    """Counter must use counter MetricPoint values."""
    STATE_SET: _MetricType.ValueType  # 3
    """State set must use state set MetricPoint values."""
    INFO: _MetricType.ValueType  # 4
    """Info must use info MetricPoint values."""
    HISTOGRAM: _MetricType.ValueType  # 5
    """Histogram must use histogram value MetricPoint values."""
    GAUGE_HISTOGRAM: _MetricType.ValueType  # 6
    """Gauge histogram must use histogram value MetricPoint values."""
    SUMMARY: _MetricType.ValueType  # 7
    """Summary quantiles must use summary value MetricPoint values."""

class MetricType(_MetricType, metaclass=_MetricTypeEnumTypeWrapper):
    """The type of a Metric."""

UNKNOWN: MetricType.ValueType  # 0
"""Unknown must use unknown MetricPoint values."""
GAUGE: MetricType.ValueType  # 1
"""Gauge must use gauge MetricPoint values."""
COUNTER: MetricType.ValueType  # 2
"""Counter must use counter MetricPoint values."""
STATE_SET: MetricType.ValueType  # 3
"""State set must use state set MetricPoint values."""
INFO: MetricType.ValueType  # 4
"""Info must use info MetricPoint values."""
HISTOGRAM: MetricType.ValueType  # 5
"""Histogram must use histogram value MetricPoint values."""
GAUGE_HISTOGRAM: MetricType.ValueType  # 6
"""Gauge histogram must use histogram value MetricPoint values."""
SUMMARY: MetricType.ValueType  # 7
"""Summary quantiles must use summary value MetricPoint values."""
Global___MetricType: _TypeAlias = MetricType  # noqa: Y015

@_typing.final
class MetricSet(_message.Message):
    """The top-level container type that is encoded and sent over the wire."""

    DESCRIPTOR: _descriptor.Descriptor

    METRIC_FAMILIES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def metric_families(self) -> _containers.RepeatedCompositeFieldContainer[Global___MetricFamily]:
        """Each MetricFamily has one or more MetricPoints for a single Metric."""

    def __init__(
        self,
        *,
        metric_families: _abc.Iterable[Global___MetricFamily] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["metric_families", b"metric_families"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___MetricSet: _TypeAlias = MetricSet  # noqa: Y015

@_typing.final
class MetricFamily(_message.Message):
    """One or more Metrics for a single MetricFamily, where each Metric
    has one or more MetricPoints.
    """

    DESCRIPTOR: _descriptor.Descriptor

    NAME_FIELD_NUMBER: _builtins.int
    TYPE_FIELD_NUMBER: _builtins.int
    UNIT_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    METRICS_FIELD_NUMBER: _builtins.int
    name: _builtins.str
    """Required."""
    type: Global___MetricType.ValueType
    """Optional."""
    unit: _builtins.str
    """Optional."""
    help: _builtins.str
    """Optional."""
    @_builtins.property
    def metrics(self) -> _containers.RepeatedCompositeFieldContainer[Global___Metric]:
        """Optional."""

    def __init__(
        self,
        *,
        name: _builtins.str = ...,
        type: Global___MetricType.ValueType = ...,
        unit: _builtins.str = ...,
        help: _builtins.str = ...,
        metrics: _abc.Iterable[Global___Metric] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["help", b"help", "metrics", b"metrics", "name", b"name", "type", b"type", "unit", b"unit"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___MetricFamily: _TypeAlias = MetricFamily  # noqa: Y015

@_typing.final
class Metric(_message.Message):
    """A single metric with a unique set of labels within a metric family."""

    DESCRIPTOR: _descriptor.Descriptor

    LABELS_FIELD_NUMBER: _builtins.int
    METRIC_POINTS_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def labels(self) -> _containers.RepeatedCompositeFieldContainer[Global___Label]:
        """Optional."""

    @_builtins.property
    def metric_points(self) -> _containers.RepeatedCompositeFieldContainer[Global___MetricPoint]:
        """Optional."""

    def __init__(
        self,
        *,
        labels: _abc.Iterable[Global___Label] | None = ...,
        metric_points: _abc.Iterable[Global___MetricPoint] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["labels", b"labels", "metric_points", b"metric_points"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Metric: _TypeAlias = Metric  # noqa: Y015

@_typing.final
class Label(_message.Message):
    """A name-value pair. These are used in multiple places: identifying
    timeseries, value of INFO metrics, and exemplars in Histograms.
    """

    DESCRIPTOR: _descriptor.Descriptor

    NAME_FIELD_NUMBER: _builtins.int
    VALUE_FIELD_NUMBER: _builtins.int
    name: _builtins.str
    """Required."""
    value: _builtins.str
    """Required."""
    def __init__(
        self,
        *,
        name: _builtins.str = ...,
        value: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["name", b"name", "value", b"value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Label: _TypeAlias = Label  # noqa: Y015

@_typing.final
class MetricPoint(_message.Message):
    """A MetricPoint in a Metric."""

    DESCRIPTOR: _descriptor.Descriptor

    UNKNOWN_VALUE_FIELD_NUMBER: _builtins.int
    GAUGE_VALUE_FIELD_NUMBER: _builtins.int
    COUNTER_VALUE_FIELD_NUMBER: _builtins.int
    HISTOGRAM_VALUE_FIELD_NUMBER: _builtins.int
    STATE_SET_VALUE_FIELD_NUMBER: _builtins.int
    INFO_VALUE_FIELD_NUMBER: _builtins.int
    SUMMARY_VALUE_FIELD_NUMBER: _builtins.int
    TIMESTAMP_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def unknown_value(self) -> Global___UnknownValue: ...
    @_builtins.property
    def gauge_value(self) -> Global___GaugeValue: ...
    @_builtins.property
    def counter_value(self) -> Global___CounterValue: ...
    @_builtins.property
    def histogram_value(self) -> Global___HistogramValue: ...
    @_builtins.property
    def state_set_value(self) -> Global___StateSetValue: ...
    @_builtins.property
    def info_value(self) -> Global___InfoValue: ...
    @_builtins.property
    def summary_value(self) -> Global___SummaryValue: ...
    @_builtins.property
    def timestamp(self) -> _timestamp_pb2.Timestamp:
        """Optional."""

    def __init__(
        self,
        *,
        unknown_value: Global___UnknownValue | None = ...,
        gauge_value: Global___GaugeValue | None = ...,
        counter_value: Global___CounterValue | None = ...,
        histogram_value: Global___HistogramValue | None = ...,
        state_set_value: Global___StateSetValue | None = ...,
        info_value: Global___InfoValue | None = ...,
        summary_value: Global___SummaryValue | None = ...,
        timestamp: _timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["counter_value", b"counter_value", "gauge_value", b"gauge_value", "histogram_value", b"histogram_value", "info_value", b"info_value", "state_set_value", b"state_set_value", "summary_value", b"summary_value", "timestamp", b"timestamp", "unknown_value", b"unknown_value", "value", b"value"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["counter_value", b"counter_value", "gauge_value", b"gauge_value", "histogram_value", b"histogram_value", "info_value", b"info_value", "state_set_value", b"state_set_value", "summary_value", b"summary_value", "timestamp", b"timestamp", "unknown_value", b"unknown_value", "value", b"value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_value: _TypeAlias = _typing.Literal["unknown_value", "gauge_value", "counter_value", "histogram_value", "state_set_value", "info_value", "summary_value"]  # noqa: Y015
    _WhichOneofArgType_value: _TypeAlias = _typing.Literal["value", b"value"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_value) -> _WhichOneofReturnType_value | None: ...

Global___MetricPoint: _TypeAlias = MetricPoint  # noqa: Y015

@_typing.final
class UnknownValue(_message.Message):
    """Value for UNKNOWN MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    DOUBLE_VALUE_FIELD_NUMBER: _builtins.int
    INT_VALUE_FIELD_NUMBER: _builtins.int
    double_value: _builtins.float
    int_value: _builtins.int
    def __init__(
        self,
        *,
        double_value: _builtins.float = ...,
        int_value: _builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["double_value", b"double_value", "int_value", b"int_value", "value", b"value"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["double_value", b"double_value", "int_value", b"int_value", "value", b"value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_value: _TypeAlias = _typing.Literal["double_value", "int_value"]  # noqa: Y015
    _WhichOneofArgType_value: _TypeAlias = _typing.Literal["value", b"value"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_value) -> _WhichOneofReturnType_value | None: ...

Global___UnknownValue: _TypeAlias = UnknownValue  # noqa: Y015

@_typing.final
class GaugeValue(_message.Message):
    """Value for GAUGE MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    DOUBLE_VALUE_FIELD_NUMBER: _builtins.int
    INT_VALUE_FIELD_NUMBER: _builtins.int
    double_value: _builtins.float
    int_value: _builtins.int
    def __init__(
        self,
        *,
        double_value: _builtins.float = ...,
        int_value: _builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["double_value", b"double_value", "int_value", b"int_value", "value", b"value"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["double_value", b"double_value", "int_value", b"int_value", "value", b"value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_value: _TypeAlias = _typing.Literal["double_value", "int_value"]  # noqa: Y015
    _WhichOneofArgType_value: _TypeAlias = _typing.Literal["value", b"value"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_value) -> _WhichOneofReturnType_value | None: ...

Global___GaugeValue: _TypeAlias = GaugeValue  # noqa: Y015

@_typing.final
class CounterValue(_message.Message):
    """Value for COUNTER MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    DOUBLE_VALUE_FIELD_NUMBER: _builtins.int
    INT_VALUE_FIELD_NUMBER: _builtins.int
    CREATED_FIELD_NUMBER: _builtins.int
    EXEMPLAR_FIELD_NUMBER: _builtins.int
    double_value: _builtins.float
    int_value: _builtins.int
    @_builtins.property
    def created(self) -> _timestamp_pb2.Timestamp:
        """The time values began being collected for this counter.
        Optional.
        """

    @_builtins.property
    def exemplar(self) -> Global___Exemplar:
        """Optional."""

    def __init__(
        self,
        *,
        double_value: _builtins.float = ...,
        int_value: _builtins.int = ...,
        created: _timestamp_pb2.Timestamp | None = ...,
        exemplar: Global___Exemplar | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["created", b"created", "double_value", b"double_value", "exemplar", b"exemplar", "int_value", b"int_value", "total", b"total"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["created", b"created", "double_value", b"double_value", "exemplar", b"exemplar", "int_value", b"int_value", "total", b"total"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_total: _TypeAlias = _typing.Literal["double_value", "int_value"]  # noqa: Y015
    _WhichOneofArgType_total: _TypeAlias = _typing.Literal["total", b"total"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_total) -> _WhichOneofReturnType_total | None: ...

Global___CounterValue: _TypeAlias = CounterValue  # noqa: Y015

@_typing.final
class HistogramValue(_message.Message):
    """Value for HISTOGRAM or GAUGE_HISTOGRAM MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    @_typing.final
    class Bucket(_message.Message):
        """Bucket is the number of values for a bucket in the histogram
        with an optional exemplar.
        """

        DESCRIPTOR: _descriptor.Descriptor

        COUNT_FIELD_NUMBER: _builtins.int
        UPPER_BOUND_FIELD_NUMBER: _builtins.int
        EXEMPLAR_FIELD_NUMBER: _builtins.int
        count: _builtins.int
        """Required."""
        upper_bound: _builtins.float
        """Optional."""
        @_builtins.property
        def exemplar(self) -> Global___Exemplar:
            """Optional."""

        def __init__(
            self,
            *,
            count: _builtins.int = ...,
            upper_bound: _builtins.float = ...,
            exemplar: Global___Exemplar | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["exemplar", b"exemplar"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["count", b"count", "exemplar", b"exemplar", "upper_bound", b"upper_bound"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    DOUBLE_VALUE_FIELD_NUMBER: _builtins.int
    INT_VALUE_FIELD_NUMBER: _builtins.int
    COUNT_FIELD_NUMBER: _builtins.int
    CREATED_FIELD_NUMBER: _builtins.int
    BUCKETS_FIELD_NUMBER: _builtins.int
    double_value: _builtins.float
    int_value: _builtins.int
    count: _builtins.int
    """Optional."""
    @_builtins.property
    def created(self) -> _timestamp_pb2.Timestamp:
        """The time values began being collected for this histogram.
        Optional.
        """

    @_builtins.property
    def buckets(self) -> _containers.RepeatedCompositeFieldContainer[Global___HistogramValue.Bucket]:
        """Optional."""

    def __init__(
        self,
        *,
        double_value: _builtins.float = ...,
        int_value: _builtins.int = ...,
        count: _builtins.int = ...,
        created: _timestamp_pb2.Timestamp | None = ...,
        buckets: _abc.Iterable[Global___HistogramValue.Bucket] | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["created", b"created", "double_value", b"double_value", "int_value", b"int_value", "sum", b"sum"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["buckets", b"buckets", "count", b"count", "created", b"created", "double_value", b"double_value", "int_value", b"int_value", "sum", b"sum"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_sum: _TypeAlias = _typing.Literal["double_value", "int_value"]  # noqa: Y015
    _WhichOneofArgType_sum: _TypeAlias = _typing.Literal["sum", b"sum"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_sum) -> _WhichOneofReturnType_sum | None: ...

Global___HistogramValue: _TypeAlias = HistogramValue  # noqa: Y015

@_typing.final
class Exemplar(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    VALUE_FIELD_NUMBER: _builtins.int
    TIMESTAMP_FIELD_NUMBER: _builtins.int
    LABEL_FIELD_NUMBER: _builtins.int
    value: _builtins.float
    """Required."""
    @_builtins.property
    def timestamp(self) -> _timestamp_pb2.Timestamp:
        """Optional."""

    @_builtins.property
    def label(self) -> _containers.RepeatedCompositeFieldContainer[Global___Label]:
        """Labels are additional information about the exemplar value (e.g. trace id).
        Optional.
        """

    def __init__(
        self,
        *,
        value: _builtins.float = ...,
        timestamp: _timestamp_pb2.Timestamp | None = ...,
        label: _abc.Iterable[Global___Label] | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["timestamp", b"timestamp"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["label", b"label", "timestamp", b"timestamp", "value", b"value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Exemplar: _TypeAlias = Exemplar  # noqa: Y015

@_typing.final
class StateSetValue(_message.Message):
    """Value for STATE_SET MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    @_typing.final
    class State(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        ENABLED_FIELD_NUMBER: _builtins.int
        NAME_FIELD_NUMBER: _builtins.int
        enabled: _builtins.bool
        """Required."""
        name: _builtins.str
        """Required."""
        def __init__(
            self,
            *,
            enabled: _builtins.bool = ...,
            name: _builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["enabled", b"enabled", "name", b"name"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    STATES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def states(self) -> _containers.RepeatedCompositeFieldContainer[Global___StateSetValue.State]:
        """Optional."""

    def __init__(
        self,
        *,
        states: _abc.Iterable[Global___StateSetValue.State] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["states", b"states"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___StateSetValue: _TypeAlias = StateSetValue  # noqa: Y015

@_typing.final
class InfoValue(_message.Message):
    """Value for INFO MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    INFO_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def info(self) -> _containers.RepeatedCompositeFieldContainer[Global___Label]:
        """Optional."""

    def __init__(
        self,
        *,
        info: _abc.Iterable[Global___Label] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["info", b"info"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___InfoValue: _TypeAlias = InfoValue  # noqa: Y015

@_typing.final
class SummaryValue(_message.Message):
    """Value for SUMMARY MetricPoint."""

    DESCRIPTOR: _descriptor.Descriptor

    @_typing.final
    class Quantile(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        QUANTILE_FIELD_NUMBER: _builtins.int
        VALUE_FIELD_NUMBER: _builtins.int
        quantile: _builtins.float
        """Required."""
        value: _builtins.float
        """Required."""
        def __init__(
            self,
            *,
            quantile: _builtins.float = ...,
            value: _builtins.float = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["quantile", b"quantile", "value", b"value"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    DOUBLE_VALUE_FIELD_NUMBER: _builtins.int
    INT_VALUE_FIELD_NUMBER: _builtins.int
    COUNT_FIELD_NUMBER: _builtins.int
    CREATED_FIELD_NUMBER: _builtins.int
    QUANTILE_FIELD_NUMBER: _builtins.int
    double_value: _builtins.float
    int_value: _builtins.int
    count: _builtins.int
    """Optional."""
    @_builtins.property
    def created(self) -> _timestamp_pb2.Timestamp:
        """The time sum and count values began being collected for this summary.
        Optional.
        """

    @_builtins.property
    def quantile(self) -> _containers.RepeatedCompositeFieldContainer[Global___SummaryValue.Quantile]:
        """Optional."""

    def __init__(
        self,
        *,
        double_value: _builtins.float = ...,
        int_value: _builtins.int = ...,
        count: _builtins.int = ...,
        created: _timestamp_pb2.Timestamp | None = ...,
        quantile: _abc.Iterable[Global___SummaryValue.Quantile] | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["created", b"created", "double_value", b"double_value", "int_value", b"int_value", "sum", b"sum"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["count", b"count", "created", b"created", "double_value", b"double_value", "int_value", b"int_value", "quantile", b"quantile", "sum", b"sum"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_sum: _TypeAlias = _typing.Literal["double_value", "int_value"]  # noqa: Y015
    _WhichOneofArgType_sum: _TypeAlias = _typing.Literal["sum", b"sum"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_sum) -> _WhichOneofReturnType_sum | None: ...

Global___SummaryValue: _TypeAlias = SummaryValue  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import ArrowData_pb2 as _ArrowData_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Dataframe(_message.Message):
    """Proto message for st.dataframe and st.data_editor elements."""

    DESCRIPTOR: _descriptor.Descriptor

    class _EditingMode:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _EditingModeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Dataframe._EditingMode.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        READ_ONLY: Dataframe._EditingMode.ValueType  # 0
        """Read-only table."""
        FIXED: Dataframe._EditingMode.ValueType  # 1
        """Activates editing but only allow editing of existing cells."""
        DYNAMIC: Dataframe._EditingMode.ValueType  # 2
        """Activates editing and allow adding & deleting rows."""
        ADD_ONLY: Dataframe._EditingMode.ValueType  # 3
        """Activates editing and allow adding rows (no deleting)."""
        DELETE_ONLY: Dataframe._EditingMode.ValueType  # 4
        """Activates editing and allow deleting rows (no adding)."""

    class EditingMode(_EditingMode, metaclass=_EditingModeEnumTypeWrapper):
        """Available editing modes:"""

    READ_ONLY: Dataframe.EditingMode.ValueType  # 0
    """Read-only table."""
    FIXED: Dataframe.EditingMode.ValueType  # 1
    """Activates editing but only allow editing of existing cells."""
    DYNAMIC: Dataframe.EditingMode.ValueType  # 2
    """Activates editing and allow adding & deleting rows."""
    ADD_ONLY: Dataframe.EditingMode.ValueType  # 3
    """Activates editing and allow adding rows (no deleting)."""
    DELETE_ONLY: Dataframe.EditingMode.ValueType  # 4
    """Activates editing and allow deleting rows (no adding)."""

    class _SelectionMode:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _SelectionModeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Dataframe._SelectionMode.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        SINGLE_ROW: Dataframe._SelectionMode.ValueType  # 0
        """Only one row can be selected at a time."""
        MULTI_ROW: Dataframe._SelectionMode.ValueType  # 1
        """Multiple rows can be selected at a time."""
        SINGLE_COLUMN: Dataframe._SelectionMode.ValueType  # 2
        """Only one column can be selected at a time."""
        MULTI_COLUMN: Dataframe._SelectionMode.ValueType  # 3
        """Multiple columns can be selected at a time."""
        SINGLE_CELL: Dataframe._SelectionMode.ValueType  # 4
        """Only one cell can be selected at a time."""
        MULTI_CELL: Dataframe._SelectionMode.ValueType  # 5
        """Multiple cells (continuous range) can be selected."""

    class SelectionMode(_SelectionMode, metaclass=_SelectionModeEnumTypeWrapper):
        """Available selection modes:"""

    SINGLE_ROW: Dataframe.SelectionMode.ValueType  # 0
    """Only one row can be selected at a time."""
    MULTI_ROW: Dataframe.SelectionMode.ValueType  # 1
    """Multiple rows can be selected at a time."""
    SINGLE_COLUMN: Dataframe.SelectionMode.ValueType  # 2
    """Only one column can be selected at a time."""
    MULTI_COLUMN: Dataframe.SelectionMode.ValueType  # 3
    """Multiple columns can be selected at a time."""
    SINGLE_CELL: Dataframe.SelectionMode.ValueType  # 4
    """Only one cell can be selected at a time."""
    MULTI_CELL: Dataframe.SelectionMode.ValueType  # 5
    """Multiple cells (continuous range) can be selected."""

    ARROW_DATA_FIELD_NUMBER: _builtins.int
    ID_FIELD_NUMBER: _builtins.int
    COLUMNS_FIELD_NUMBER: _builtins.int
    EDITING_MODE_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    COLUMN_ORDER_FIELD_NUMBER: _builtins.int
    SELECTION_MODE_FIELD_NUMBER: _builtins.int
    ROW_HEIGHT_FIELD_NUMBER: _builtins.int
    PLACEHOLDER_FIELD_NUMBER: _builtins.int
    SELECTION_STATE_FIELD_NUMBER: _builtins.int
    SELECTION_DEFAULT_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    """The id of the widget, this is required if the dataframe is editable"""
    columns: _builtins.str
    """Column configuration as JSON"""
    editing_mode: Global___Dataframe.EditingMode.ValueType
    """Activate table editing"""
    disabled: _builtins.bool
    """Deactivates editing"""
    form_id: _builtins.str
    """The form ID of the widget, this is required if the dataframe is editable"""
    row_height: _builtins.int
    """Row height in pixels"""
    placeholder: _builtins.str
    """Placeholder string used when rendering missing values (e.g. nulls, NaNs)."""
    selection_state: _builtins.str
    """Selection state (JSON serialized), set when selection is changed programmatically
    via session state. When present, frontend applies this selection.
    When absent, frontend manages its own selection state.
    """
    selection_default: _builtins.str
    """Default selection state (JSON serialized), set when selection_default is provided.
    Applied by the frontend only when there is no stored selection state.
    """
    @_builtins.property
    def arrow_data(self) -> _ArrowData_pb2.ArrowData:
        """The Arrow data to display"""

    @_builtins.property
    def column_order(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]:
        """Defines the order in which columns are displayed"""

    @_builtins.property
    def selection_mode(self) -> _containers.RepeatedScalarFieldContainer[Global___Dataframe.SelectionMode.ValueType]:
        """Activated dataframe selections events"""

    def __init__(
        self,
        *,
        arrow_data: _ArrowData_pb2.ArrowData | None = ...,
        id: _builtins.str = ...,
        columns: _builtins.str = ...,
        editing_mode: Global___Dataframe.EditingMode.ValueType = ...,
        disabled: _builtins.bool = ...,
        form_id: _builtins.str = ...,
        column_order: _abc.Iterable[_builtins.str] | None = ...,
        selection_mode: _abc.Iterable[Global___Dataframe.SelectionMode.ValueType] | None = ...,
        row_height: _builtins.int | None = ...,
        placeholder: _builtins.str | None = ...,
        selection_state: _builtins.str | None = ...,
        selection_default: _builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_placeholder", b"_placeholder", "_row_height", b"_row_height", "_selection_default", b"_selection_default", "_selection_state", b"_selection_state", "arrow_data", b"arrow_data", "placeholder", b"placeholder", "row_height", b"row_height", "selection_default", b"selection_default", "selection_state", b"selection_state"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_placeholder", b"_placeholder", "_row_height", b"_row_height", "_selection_default", b"_selection_default", "_selection_state", b"_selection_state", "arrow_data", b"arrow_data", "column_order", b"column_order", "columns", b"columns", "disabled", b"disabled", "editing_mode", b"editing_mode", "form_id", b"form_id", "id", b"id", "placeholder", b"placeholder", "row_height", b"row_height", "selection_default", b"selection_default", "selection_mode", b"selection_mode", "selection_state", b"selection_state"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__placeholder: _TypeAlias = _typing.Literal["placeholder"]  # noqa: Y015
    _WhichOneofArgType__placeholder: _TypeAlias = _typing.Literal["_placeholder", b"_placeholder"]  # noqa: Y015
    _WhichOneofReturnType__row_height: _TypeAlias = _typing.Literal["row_height"]  # noqa: Y015
    _WhichOneofArgType__row_height: _TypeAlias = _typing.Literal["_row_height", b"_row_height"]  # noqa: Y015
    _WhichOneofReturnType__selection_default: _TypeAlias = _typing.Literal["selection_default"]  # noqa: Y015
    _WhichOneofArgType__selection_default: _TypeAlias = _typing.Literal["_selection_default", b"_selection_default"]  # noqa: Y015
    _WhichOneofReturnType__selection_state: _TypeAlias = _typing.Literal["selection_state"]  # noqa: Y015
    _WhichOneofArgType__selection_state: _TypeAlias = _typing.Literal["_selection_state", b"_selection_state"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__placeholder) -> _WhichOneofReturnType__placeholder | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__row_height) -> _WhichOneofReturnType__row_height | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__selection_default) -> _WhichOneofReturnType__selection_default | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__selection_state) -> _WhichOneofReturnType__selection_state | None: ...

Global___Dataframe: _TypeAlias = Dataframe  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import AuthRedirect_pb2 as _AuthRedirect_pb2
from streamlit.proto import AutoRerun_pb2 as _AutoRerun_pb2
from streamlit.proto import Common_pb2 as _Common_pb2
from streamlit.proto import Delta_pb2 as _Delta_pb2
from streamlit.proto import GitInfo_pb2 as _GitInfo_pb2
from streamlit.proto import Logo_pb2 as _Logo_pb2
from streamlit.proto import Navigation_pb2 as _Navigation_pb2
from streamlit.proto import NewSession_pb2 as _NewSession_pb2
from streamlit.proto import PageConfig_pb2 as _PageConfig_pb2
from streamlit.proto import PageInfo_pb2 as _PageInfo_pb2
from streamlit.proto import PageNotFound_pb2 as _PageNotFound_pb2
from streamlit.proto import PageProfile_pb2 as _PageProfile_pb2
from streamlit.proto import ParentMessage_pb2 as _ParentMessage_pb2
from streamlit.proto import SessionEvent_pb2 as _SessionEvent_pb2
from streamlit.proto import SessionStatus_pb2 as _SessionStatus_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class ForwardMsg(_message.Message):
    """A message sent from Proxy to the browser"""

    DESCRIPTOR: _descriptor.Descriptor

    class _ScriptFinishedStatus:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _ScriptFinishedStatusEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[ForwardMsg._ScriptFinishedStatus.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        FINISHED_SUCCESSFULLY: ForwardMsg._ScriptFinishedStatus.ValueType  # 0
        """The script compiled and ran."""
        FINISHED_WITH_COMPILE_ERROR: ForwardMsg._ScriptFinishedStatus.ValueType  # 1
        """The script failed to compile"""
        FINISHED_EARLY_FOR_RERUN: ForwardMsg._ScriptFinishedStatus.ValueType  # 2
        """Script was interrupted by rerun"""
        FINISHED_FRAGMENT_RUN_SUCCESSFULLY: ForwardMsg._ScriptFinishedStatus.ValueType  # 3
        """A fragment of the script ran successfully."""

    class ScriptFinishedStatus(_ScriptFinishedStatus, metaclass=_ScriptFinishedStatusEnumTypeWrapper):
        """Values for the 'script_finished` type"""

    FINISHED_SUCCESSFULLY: ForwardMsg.ScriptFinishedStatus.ValueType  # 0
    """The script compiled and ran."""
    FINISHED_WITH_COMPILE_ERROR: ForwardMsg.ScriptFinishedStatus.ValueType  # 1
    """The script failed to compile"""
    FINISHED_EARLY_FOR_RERUN: ForwardMsg.ScriptFinishedStatus.ValueType  # 2
    """Script was interrupted by rerun"""
    FINISHED_FRAGMENT_RUN_SUCCESSFULLY: ForwardMsg.ScriptFinishedStatus.ValueType  # 3
    """A fragment of the script ran successfully."""

    HASH_FIELD_NUMBER: _builtins.int
    METADATA_FIELD_NUMBER: _builtins.int
    NEW_SESSION_FIELD_NUMBER: _builtins.int
    DELTA_FIELD_NUMBER: _builtins.int
    PAGE_INFO_CHANGED_FIELD_NUMBER: _builtins.int
    PAGE_CONFIG_CHANGED_FIELD_NUMBER: _builtins.int
    SCRIPT_FINISHED_FIELD_NUMBER: _builtins.int
    GIT_INFO_CHANGED_FIELD_NUMBER: _builtins.int
    PAGE_PROFILE_FIELD_NUMBER: _builtins.int
    SESSION_STATUS_CHANGED_FIELD_NUMBER: _builtins.int
    SESSION_EVENT_FIELD_NUMBER: _builtins.int
    NAVIGATION_FIELD_NUMBER: _builtins.int
    PAGE_NOT_FOUND_FIELD_NUMBER: _builtins.int
    FILE_URLS_RESPONSE_FIELD_NUMBER: _builtins.int
    AUTO_RERUN_FIELD_NUMBER: _builtins.int
    LOGO_FIELD_NUMBER: _builtins.int
    AUTH_REDIRECT_FIELD_NUMBER: _builtins.int
    PARENT_MESSAGE_FIELD_NUMBER: _builtins.int
    REF_HASH_FIELD_NUMBER: _builtins.int
    DEFERRED_FILE_RESPONSE_FIELD_NUMBER: _builtins.int
    HEARTBEAT_ACK_FIELD_NUMBER: _builtins.int
    DEBUG_LAST_BACKMSG_ID_FIELD_NUMBER: _builtins.int
    hash: _builtins.str
    """A hash that uniquely identifies this ForwardMsg, for caching."""
    script_finished: Global___ForwardMsg.ScriptFinishedStatus.ValueType
    ref_hash: _builtins.str
    """A reference to a ForwardMsg that has already been delivered
    and cached in the frontend. The client should substitute the message
    with the given hash for this one.
    """
    heartbeat_ack: _builtins.bool
    """Acknowledgment of app_heartbeat BackMsg, used for connection health
    monitoring. When the frontend sends an app_heartbeat, the server
    responds with this ack so the frontend can verify the connection is
    healthy.
    """
    debug_last_backmsg_id: _builtins.str
    """The ID of the last BackMsg that we received before sending this
    ForwardMsg. As its name suggests, this field should only be used for
    testing.
    """
    @_builtins.property
    def metadata(self) -> Global___ForwardMsgMetadata:
        """Contains 'non-payload' ForwardMsg data that isn't cached for the purposes
        of ForwardMsg de-duping.
        """

    @_builtins.property
    def new_session(self) -> _NewSession_pb2.NewSession:
        """App lifecycle messages."""

    @_builtins.property
    def delta(self) -> _Delta_pb2.Delta: ...
    @_builtins.property
    def page_info_changed(self) -> _PageInfo_pb2.PageInfo: ...
    @_builtins.property
    def page_config_changed(self) -> _PageConfig_pb2.PageConfig: ...
    @_builtins.property
    def git_info_changed(self) -> _GitInfo_pb2.GitInfo: ...
    @_builtins.property
    def page_profile(self) -> _PageProfile_pb2.PageProfile: ...
    @_builtins.property
    def session_status_changed(self) -> _SessionStatus_pb2.SessionStatus:
        """Status change and event messages."""

    @_builtins.property
    def session_event(self) -> _SessionEvent_pb2.SessionEvent: ...
    @_builtins.property
    def navigation(self) -> _Navigation_pb2.Navigation:
        """Other messages."""

    @_builtins.property
    def page_not_found(self) -> _PageNotFound_pb2.PageNotFound: ...
    @_builtins.property
    def file_urls_response(self) -> _Common_pb2.FileURLsResponse: ...
    @_builtins.property
    def auto_rerun(self) -> _AutoRerun_pb2.AutoRerun: ...
    @_builtins.property
    def logo(self) -> _Logo_pb2.Logo:
        """App logo message"""

    @_builtins.property
    def auth_redirect(self) -> _AuthRedirect_pb2.AuthRedirect:
        """Auth redirect message"""

    @_builtins.property
    def parent_message(self) -> _ParentMessage_pb2.ParentMessage:
        """Platform - message to host"""

    @_builtins.property
    def deferred_file_response(self) -> Global___DeferredFileResponse:
        """Response for a deferred file request"""

    def __init__(
        self,
        *,
        hash: _builtins.str = ...,
        metadata: Global___ForwardMsgMetadata | None = ...,
        new_session: _NewSession_pb2.NewSession | None = ...,
        delta: _Delta_pb2.Delta | None = ...,
        page_info_changed: _PageInfo_pb2.PageInfo | None = ...,
        page_config_changed: _PageConfig_pb2.PageConfig | None = ...,
        script_finished: Global___ForwardMsg.ScriptFinishedStatus.ValueType = ...,
        git_info_changed: _GitInfo_pb2.GitInfo | None = ...,
        page_profile: _PageProfile_pb2.PageProfile | None = ...,
        session_status_changed: _SessionStatus_pb2.SessionStatus | None = ...,
        session_event: _SessionEvent_pb2.SessionEvent | None = ...,
        navigation: _Navigation_pb2.Navigation | None = ...,
        page_not_found: _PageNotFound_pb2.PageNotFound | None = ...,
        file_urls_response: _Common_pb2.FileURLsResponse | None = ...,
        auto_rerun: _AutoRerun_pb2.AutoRerun | None = ...,
        logo: _Logo_pb2.Logo | None = ...,
        auth_redirect: _AuthRedirect_pb2.AuthRedirect | None = ...,
        parent_message: _ParentMessage_pb2.ParentMessage | None = ...,
        ref_hash: _builtins.str = ...,
        deferred_file_response: Global___DeferredFileResponse | None = ...,
        heartbeat_ack: _builtins.bool = ...,
        debug_last_backmsg_id: _builtins.str = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["auth_redirect", b"auth_redirect", "auto_rerun", b"auto_rerun", "deferred_file_response", b"deferred_file_response", "delta", b"delta", "file_urls_response", b"file_urls_response", "git_info_changed", b"git_info_changed", "heartbeat_ack", b"heartbeat_ack", "logo", b"logo", "metadata", b"metadata", "navigation", b"navigation", "new_session", b"new_session", "page_config_changed", b"page_config_changed", "page_info_changed", b"page_info_changed", "page_not_found", b"page_not_found", "page_profile", b"page_profile", "parent_message", b"parent_message", "ref_hash", b"ref_hash", "script_finished", b"script_finished", "session_event", b"session_event", "session_status_changed", b"session_status_changed", "type", b"type"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["auth_redirect", b"auth_redirect", "auto_rerun", b"auto_rerun", "debug_last_backmsg_id", b"debug_last_backmsg_id", "deferred_file_response", b"deferred_file_response", "delta", b"delta", "file_urls_response", b"file_urls_response", "git_info_changed", b"git_info_changed", "hash", b"hash", "heartbeat_ack", b"heartbeat_ack", "logo", b"logo", "metadata", b"metadata", "navigation", b"navigation", "new_session", b"new_session", "page_config_changed", b"page_config_changed", "page_info_changed", b"page_info_changed", "page_not_found", b"page_not_found", "page_profile", b"page_profile", "parent_message", b"parent_message", "ref_hash", b"ref_hash", "script_finished", b"script_finished", "session_event", b"session_event", "session_status_changed", b"session_status_changed", "type", b"type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["new_session", "delta", "page_info_changed", "page_config_changed", "script_finished", "git_info_changed", "page_profile", "session_status_changed", "session_event", "navigation", "page_not_found", "file_urls_response", "auto_rerun", "logo", "auth_redirect", "parent_message", "ref_hash", "deferred_file_response", "heartbeat_ack"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___ForwardMsg: _TypeAlias = ForwardMsg  # noqa: Y015

@_typing.final
class DeferredFileResponse(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    FILE_ID_FIELD_NUMBER: _builtins.int
    URL_FIELD_NUMBER: _builtins.int
    ERROR_MSG_FIELD_NUMBER: _builtins.int
    file_id: _builtins.str
    url: _builtins.str
    error_msg: _builtins.str
    def __init__(
        self,
        *,
        file_id: _builtins.str = ...,
        url: _builtins.str = ...,
        error_msg: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["error_msg", b"error_msg", "file_id", b"file_id", "url", b"url"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DeferredFileResponse: _TypeAlias = DeferredFileResponse  # noqa: Y015

@_typing.final
class ForwardMsgMetadata(_message.Message):
    """ForwardMsgMetadata contains all data that does _not_ get hashed (or cached)
    in our ForwardMsgCache. (That is, when we cache a ForwardMsg, we clear its
    metadata field first.) This allows us to, e.g., have a large unchanging
    dataframe appear in different places across multiple reruns - or even appear
    multiple times in a single run - and only send its dataframe bytes once.
    """

    DESCRIPTOR: _descriptor.Descriptor

    CACHEABLE_FIELD_NUMBER: _builtins.int
    DELTA_PATH_FIELD_NUMBER: _builtins.int
    ACTIVE_SCRIPT_HASH_FIELD_NUMBER: _builtins.int
    cacheable: _builtins.bool
    """Marks a message as cacheable for the frontend."""
    active_script_hash: _builtins.str
    """active_script_hash the forward message is associated from.
    For multipage apps v1, this will always be the page file running
    For multipage apps v2, this can be the main script or the page script
    """
    @_builtins.property
    def delta_path(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]:
        """The path that identifies a delta's location in the report tree.
        Only set for Delta messages.
        """

    def __init__(
        self,
        *,
        cacheable: _builtins.bool = ...,
        delta_path: _abc.Iterable[_builtins.int] | None = ...,
        active_script_hash: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["active_script_hash", b"active_script_hash", "cacheable", b"cacheable", "delta_path", b"delta_path"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ForwardMsgMetadata: _TypeAlias = ForwardMsgMetadata  # noqa: Y015

@_typing.final
class ForwardMsgList(_message.Message):
    """This is a list of ForwardMessages used to have a single protobuf message
    that encapsulates multiple ForwardMessages. This is used in static streamlit app
    generation in replaying all of the protobuf messages of a streamlit app. The
    ForwardMsgList allows us to leverage the built-ins of protobuf in delimiting the ForwardMsgs
    instead of needing to do that ourselves.
    """

    DESCRIPTOR: _descriptor.Descriptor

    MESSAGES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def messages(self) -> _containers.RepeatedCompositeFieldContainer[Global___ForwardMsg]: ...
    def __init__(
        self,
        *,
        messages: _abc.Iterable[Global___ForwardMsg] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["messages", b"messages"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ForwardMsgList: _TypeAlias = ForwardMsgList  # noqa: Y015



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import json
from typing import TYPE_CHECKING, Any

from authlib.integrations.base_client import (
    FrameworkIntegration,
)

from streamlit.runtime.secrets import AttrDict

if TYPE_CHECKING:
    from collections.abc import Sequence

    from streamlit.web.server.oidc_mixin import TornadoOAuth


class TornadoIntegration(FrameworkIntegration):
    """Authlib framework integration for Tornado.

    Authlib always tracks state in sessions (post 1.6.6+), but Tornado doesn't have persistent
    sessions like Flask/Django, so we override the state data methods to use only the cache.
    This is compatible with both older authlib versions and authlib 1.6.6+.
    """

    def _get_cache_data(self, key: str) -> dict[str, Any] | None:
        """Get and decode JSON data from cache.

        This reimplements the cache data retrieval logic directly rather than
        relying on authlib's private _get_cache_data method, ensuring stability
        across authlib versions.
        """
        value = self.cache.get(key)
        if not value:
            return None
        try:
            result = json.loads(value)
            if isinstance(result, dict):
                return result
            return None
        except (TypeError, ValueError):
            return None

    def get_state_data(
        self,
        session: dict[str, Any],  # noqa: ARG002
        state: str,
    ) -> dict[str, Any] | None:
        """Get OAuth state data from cache.

        Overridden to use cache directly without requiring session persistence,
        since Tornado doesn't have persistent sessions across requests.
        The session parameter is unused but required by the parent class signature.
        """
        if not self.cache:
            return None
        key = f"_state_{self.name}_{state}"
        cached_value = self._get_cache_data(key)
        if cached_value:
            return cached_value.get("data")
        return None

    def clear_state_data(
        self,
        session: dict[str, Any],  # noqa: ARG002
        state: str,
    ) -> None:
        """Clear OAuth state data from cache.

        Overridden to use cache directly without requiring session persistence,
        since Tornado doesn't have persistent sessions across requests.
        The session parameter is unused but required by the parent class signature.
        """
        if self.cache:
            key = f"_state_{self.name}_{state}"
            self.cache.delete(key)

    def update_token(
        self,
        token: dict[str, Any],
        refresh_token: dict[str, Any] | None = None,
        access_token: dict[str, Any] | None = None,
    ) -> None:
        """We do not support access token refresh, since we obtain and operate only on
        identity tokens. We override this method explicitly to implement all abstract
        methods of base class.
        """

    @staticmethod
    def load_config(
        oauth: TornadoOAuth, name: str, params: Sequence[str]
    ) -> dict[str, Any]:
        """Configure Authlib integration with provider parameters
        specified in secrets.toml.
        """

        # oauth.config here is an auth section from secrets.toml
        # We parse it here to transform nested AttrDict (for client_kwargs value)
        # to dict so Authlib can work with it under the hood.
        if not oauth.config:
            return {}

        prepared_config = {}
        for key in params:
            value = oauth.config.get(name, {}).get(key, None)
            if isinstance(value, AttrDict):
                # We want to modify client_kwargs further after loading server metadata
                value = value.to_dict()
            if value is not None:
                prepared_config[key] = value
        return prepared_config



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import LabelVisibility_pb2 as _LabelVisibility_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class ButtonGroup(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _ClickMode:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _ClickModeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[ButtonGroup._ClickMode.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        SINGLE_SELECT: ButtonGroup._ClickMode.ValueType  # 0
        MULTI_SELECT: ButtonGroup._ClickMode.ValueType  # 1

    class ClickMode(_ClickMode, metaclass=_ClickModeEnumTypeWrapper): ...
    SINGLE_SELECT: ButtonGroup.ClickMode.ValueType  # 0
    MULTI_SELECT: ButtonGroup.ClickMode.ValueType  # 1

    class _Style:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _StyleEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[ButtonGroup._Style.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        SEGMENTED_CONTROL: ButtonGroup._Style.ValueType  # 0
        PILLS: ButtonGroup._Style.ValueType  # 1

    class Style(_Style, metaclass=_StyleEnumTypeWrapper): ...
    SEGMENTED_CONTROL: ButtonGroup.Style.ValueType  # 0
    PILLS: ButtonGroup.Style.ValueType  # 1

    @_typing.final
    class Option(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        CONTENT_FIELD_NUMBER: _builtins.int
        CONTENT_ICON_FIELD_NUMBER: _builtins.int
        content: _builtins.str
        content_icon: _builtins.str
        """when set, this is the icon that will be displayed next to the option"""
        def __init__(
            self,
            *,
            content: _builtins.str = ...,
            content_icon: _builtins.str | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["_content_icon", b"_content_icon", "content_icon", b"content_icon"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["_content_icon", b"_content_icon", "content", b"content", "content_icon", b"content_icon"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType__content_icon: _TypeAlias = _typing.Literal["content_icon"]  # noqa: Y015
        _WhichOneofArgType__content_icon: _TypeAlias = _typing.Literal["_content_icon", b"_content_icon"]  # noqa: Y015
        def WhichOneof(self, oneof_group: _WhichOneofArgType__content_icon) -> _WhichOneofReturnType__content_icon | None: ...

    ID_FIELD_NUMBER: _builtins.int
    OPTIONS_FIELD_NUMBER: _builtins.int
    DEFAULT_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    CLICK_MODE_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    SET_VALUE_FIELD_NUMBER: _builtins.int
    STYLE_FIELD_NUMBER: _builtins.int
    LABEL_FIELD_NUMBER: _builtins.int
    LABEL_VISIBILITY_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    RAW_VALUES_FIELD_NUMBER: _builtins.int
    QUERY_PARAM_KEY_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    disabled: _builtins.bool
    click_mode: Global___ButtonGroup.ClickMode.ValueType
    form_id: _builtins.str
    set_value: _builtins.bool
    style: Global___ButtonGroup.Style.ValueType
    label: _builtins.str
    help: _builtins.str
    query_param_key: _builtins.str
    """If set, widget value is bound to this query parameter key"""
    @_builtins.property
    def options(self) -> _containers.RepeatedCompositeFieldContainer[Global___ButtonGroup.Option]: ...
    @_builtins.property
    def default(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]:
        """default is an array of indexes that are selected by default"""

    @_builtins.property
    def label_visibility(self) -> _LabelVisibility_pb2.LabelVisibility: ...
    @_builtins.property
    def raw_values(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]:
        """For multi-select mode, this contains all selected option strings.
        For single-select mode, this contains at most one selected option string.
        """

    def __init__(
        self,
        *,
        id: _builtins.str = ...,
        options: _abc.Iterable[Global___ButtonGroup.Option] | None = ...,
        default: _abc.Iterable[_builtins.int] | None = ...,
        disabled: _builtins.bool = ...,
        click_mode: Global___ButtonGroup.ClickMode.ValueType = ...,
        form_id: _builtins.str = ...,
        set_value: _builtins.bool = ...,
        style: Global___ButtonGroup.Style.ValueType = ...,
        label: _builtins.str = ...,
        label_visibility: _LabelVisibility_pb2.LabelVisibility | None = ...,
        help: _builtins.str | None = ...,
        raw_values: _abc.Iterable[_builtins.str] | None = ...,
        query_param_key: _builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_help", b"_help", "_query_param_key", b"_query_param_key", "help", b"help", "label_visibility", b"label_visibility", "query_param_key", b"query_param_key"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_help", b"_help", "_query_param_key", b"_query_param_key", "click_mode", b"click_mode", "default", b"default", "disabled", b"disabled", "form_id", b"form_id", "help", b"help", "id", b"id", "label", b"label", "label_visibility", b"label_visibility", "options", b"options", "query_param_key", b"query_param_key", "raw_values", b"raw_values", "set_value", b"set_value", "style", b"style"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__help: _TypeAlias = _typing.Literal["help"]  # noqa: Y015
    _WhichOneofArgType__help: _TypeAlias = _typing.Literal["_help", b"_help"]  # noqa: Y015
    _WhichOneofReturnType__query_param_key: _TypeAlias = _typing.Literal["query_param_key"]  # noqa: Y015
    _WhichOneofArgType__query_param_key: _TypeAlias = _typing.Literal["_query_param_key", b"_query_param_key"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__help) -> _WhichOneofReturnType__help | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__query_param_key) -> _WhichOneofReturnType__query_param_key | None: ...

Global___ButtonGroup: _TypeAlias = ButtonGroup  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import ArrowData_pb2 as _ArrowData_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Table(_message.Message):
    """Proto message for st.table element."""

    DESCRIPTOR: _descriptor.Descriptor

    class _BorderMode:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _BorderModeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Table._BorderMode.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        ALL: Table._BorderMode.ValueType  # 0
        """Show borders around the table and between cells."""
        NONE: Table._BorderMode.ValueType  # 1
        """Show no borders."""
        HORIZONTAL: Table._BorderMode.ValueType  # 2
        """Show only horizontal borders between rows."""

    class BorderMode(_BorderMode, metaclass=_BorderModeEnumTypeWrapper):
        """Available border modes:"""

    ALL: Table.BorderMode.ValueType  # 0
    """Show borders around the table and between cells."""
    NONE: Table.BorderMode.ValueType  # 1
    """Show no borders."""
    HORIZONTAL: Table.BorderMode.ValueType  # 2
    """Show only horizontal borders between rows."""

    ARROW_DATA_FIELD_NUMBER: _builtins.int
    BORDER_MODE_FIELD_NUMBER: _builtins.int
    HIDE_INDEX_FIELD_NUMBER: _builtins.int
    HIDE_HEADER_FIELD_NUMBER: _builtins.int
    border_mode: Global___Table.BorderMode.ValueType
    """How to show borders around the table and between cells."""
    hide_index: _builtins.bool
    """Whether to hide the index column(s)."""
    hide_header: _builtins.bool
    """Whether to hide the column header row(s)."""
    @_builtins.property
    def arrow_data(self) -> _ArrowData_pb2.ArrowData:
        """The Arrow data to display"""

    def __init__(
        self,
        *,
        arrow_data: _ArrowData_pb2.ArrowData | None = ...,
        border_mode: Global___Table.BorderMode.ValueType = ...,
        hide_index: _builtins.bool = ...,
        hide_header: _builtins.bool = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["arrow_data", b"arrow_data"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["arrow_data", b"arrow_data", "border_mode", b"border_mode", "hide_header", b"hide_header", "hide_index", b"hide_index"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Table: _TypeAlias = Table  # noqa: Y015
